#include "AlarmTreeWidget.h"
#include "filter.h"
#include <QStringList>
#include<QHeaderView>
#include <QApplication>
#include <QDateTime>
#include <elettracolors.h>
#include "AlarmTreeWidgetItem.h"
#include <QMouseEvent>
#include <QMenu>
#include <QAction>

using namespace std;

AlarmTreeWidget::AlarmTreeWidget(QWidget *parent) : QTreeWidget(parent)
{
	setColumnCount(9);
	setRootIsDecorated(false);
	setAutoScroll(true);
	setSelectionMode(QAbstractItemView::ExtendedSelection);
    setHeaderLabels(QStringList() << "Date/time" << "microsecs" << "Alarm" << "Status" << "Ack" << "Count" << "Level" << "Silenced min." <<
			"Group" << "Message");
	header()->setResizeMode(  QHeaderView::ResizeToContents);
	_filter_enabled = false;

    setMouseTracking(true);
    connect(this, SIGNAL(itemEntered(QTreeWidgetItem*,int)), this, SLOT(slotItemEntered(QTreeWidgetItem*,int)));
}

void AlarmTreeWidget::showEvent(QShowEvent *e)
{
  //  header()->setResizeMode(  QHeaderView::Interactive);
	QWidget::showEvent(e);
}

void AlarmTreeWidget::applyFilter(AlarmFilter f)
{
	QList<QTreeWidgetItem *> items;
	int i;
	
	if(_filter_enabled)
		removeFilter();

	items = findItems("*", Qt::MatchWildcard);

	if(f.ack_enable)
	{
		QString sack = f.ack ? "ACK" : "NACK";
		for(i = 0; i < items.size(); i++)
			if(items[i]->text(ACK) != sack)
				items[i]->setHidden(true);
		headerItem()->setIcon(ACK, QIcon(ICON_PATH + "filter.png"));
	}
	else
		headerItem()->setIcon(ACK, QIcon(ICON_PATH + ""));


	if(f.lev)
	{
		for(i = 0; i < items.size(); i++)
		{
			/* must not have been hidden previously */
			if(!items[i]->isHidden())
			{
				if(items[i]->text(LEVEL) != f.level)
					items[i]->setHidden(true);
			}
		}
		headerItem()->setIcon(LEVEL, QIcon(ICON_PATH + "filter.png"));
	}
	else
		headerItem()->setIcon(LEVEL, QIcon(ICON_PATH + ""));


	if(f.stat)
	{
		for(i = 0; i < items.size(); i++)
		{
			/* must not have been hidden previously */
			if(!items[i]->isHidden())
			{
				if(items[i]->text(STATUS) != f.status)
					items[i]->setHidden(true);
			}
		}
		headerItem()->setIcon(STATUS, QIcon(ICON_PATH + "filter.png"));
	}
	else
		headerItem()->setIcon(STATUS, QIcon(ICON_PATH + ""));


	if(f.grp)
	{
		for(i = 0; i < items.size(); i++)
		{
			/* must not have been hidden previously */
			if(!items[i]->isHidden())
			{
				for(int j = 0; j < f.groups.size(); j++)
				{
					if(!items[i]->text(GROUP).contains(f.groups[j]))
					{
						items[i]->setHidden(true);
						break;
					}
				}
			}
		}
		headerItem()->setIcon(GROUP, QIcon(ICON_PATH + "filter.png"));
	}
	else
		headerItem()->setIcon(GROUP, QIcon(ICON_PATH + ""));

	_filter_enabled = true;

}

void AlarmTreeWidget::removeFilter()
{
	QList<QTreeWidgetItem *> items;
	int i;
	items = findItems("*", Qt::MatchWildcard);
	for(i = 0; i < items.size(); i++)
		if(items[i]->isHidden())
			items[i]->setHidden(false);
	_filter_enabled = false;
	headerItem()->setIcon(ACK, QIcon(ICON_PATH + ""));
	headerItem()->setIcon(LEVEL, QIcon(ICON_PATH + ""));
	headerItem()->setIcon(STATUS, QIcon(ICON_PATH + ""));
	headerItem()->setIcon(GROUP, QIcon(ICON_PATH + ""));
}

void AlarmTreeWidget::cloneTreeWidget(QTreeWidget* it)
{
	QStringList itemS;
	clear();
	QList<QTreeWidgetItem *>items = it->findItems("*", Qt::MatchWildcard);
	for(int i = 0; i < items.size(); i++)
	{
		itemS.clear();
		for(int j =0; j < items[i]->columnCount(); j++)
			itemS << items[i]->text(j);
		QTreeWidgetItem *clone = new QTreeWidgetItem(this, itemS);
	}
}

void AlarmTreeWidget::contextMenuEvent(QContextMenuEvent *e)
{
    bool silenceAvail = true;
    QList<QTreeWidgetItem *>items = this->selectedItems();
    foreach(QTreeWidgetItem *it, items)
    {
        int sil = it->text(AlarmTreeWidgetItem::SilencedMin).toInt();
        if(sil < 0)
        {
            silenceAvail = false;
            break;
        }
    }
    QMenu *menu = new QMenu(this);
    menu->addAction("Acknowledge", this, SIGNAL(ack()));
    if(silenceAvail)
        menu->addAction("Silence", this, SIGNAL(silence()));
    menu->exec(QCursor::pos());
    delete menu;
}

void AlarmTreeWidget::mouseMoveEvent(QMouseEvent *event)
{
    QTreeWidget::mouseMoveEvent(event);
    QPoint p = event->pos();
    emit mouseOverItem(this->itemAt(p) != NULL);
}

void AlarmTreeWidget::slotItemEntered(QTreeWidgetItem* it, int)
{
    QString mes = it->text(AlarmTreeWidgetItem::Message);
    emit message(mes);
}


