#include <AlarmTreeWidgetItem.h>
#include <QStringList>
#include <QDateTime>
#include <QTreeWidget>
//#include <QtDebug>
#include <elettracolors.h>
#include <iostream>
#include <macros.h>

AlarmTreeWidgetItem::AlarmTreeWidgetItem(QTreeWidget *parent, QStringList &strings)
    : QTreeWidgetItem(parent, strings)
{
    _hide = false;

    if(strings.size() < 10)
    {
        for(int j = 0; j < 10 - strings.size(); j++)
            strings << "-";
    }

    setColor(strings.at(Status), strings.at(SilencedMin).toInt());

    mHasMutedIcon = false;
    mNullIcon  = QIcon();
    mMutedIcon = QIcon(":icons/audio-volume-muted.png");
}

AlarmTreeWidgetItem::~AlarmTreeWidgetItem()
{

}

void AlarmTreeWidgetItem::setNew(bool isNew)
{
    QFont f = font(Alarm);
    f.setBold(isNew);
    setFont(Alarm, f);
}

void AlarmTreeWidgetItem::setColor(const QString& colorField, int silencedTimeo)
{
    if(colorField.contains("ALARM"))
    {
        for(int j = 0; j < columnCount(); j++)
            setBackground(j, QColor(255,115,115));
    }
    else if(colorField.contains("WARNING"))
    {
        for(int j = 0; j < columnCount(); j++)
            setBackground(j, QBrush(EColor(Elettra::yellow)));
    }
    else
    {
        for(int j = 0; j < columnCount(); j++)
            setBackground(j, QBrush(EColor(Elettra::white)));
    }

}

void AlarmTreeWidgetItem::update(const QStringList &fields)
{
    QString colorField = fields.at(Status);
    int silenced = fields.at(SilencedMin).toInt();
    bool isNew = fields.last() == "NEW";
    for(int i = 0; i < fields.size() && i < columnCount(); i++)
    {
        QString field = fields.at(i);
        if(i == 0)
            field = QDateTime::fromTime_t(field.toUInt()).toString();
        if(text(i) != field)
            setText(i, field);
        if(colorField.contains("ALARM"))
            setBackground(i, QColor(255,115,115));
        else if(colorField.contains("WARNING"))
        {
                setBackground(i, QBrush(EColor(Elettra::yellow)));
        }
        else
        {
                setBackground(i, QBrush(EColor(Elettra::white)));
        }

    }

    QFont f = font(Alarm);
    f.setBold(isNew);
    setFont(Alarm, f);

    if(silenced > 0 && !mHasMutedIcon)
    {
        setIcon(SilencedMin, mMutedIcon);
        QColor c = background(SilencedMin).color();
        c.setAlpha(100);
        setBackground(SilencedMin, c);
        mHasMutedIcon = true;
    }
    else if(mHasMutedIcon && silenced < 0)
    {
        mHasMutedIcon = false;
        setIcon(SilencedMin, mNullIcon);
    }
}



