/***************************************************************************
*   Copyright (C) 2007 by Claudio Scafuri, Giacomo Strangolino   *
*   claudio@hyo   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#include "alarm.h"
#include "filter.h"
#include "AlarmTreeWidget.h"
#include "AlarmTreeWidgetItem.h"
#include <QMessageBox>
#include <QDateTime>
#include <QHeaderView>
#include <QCloseEvent>
#include "AlarmHistoryTree.h"
#include "systray.h"
#include <QDebug>
#include <QTWriter>
#include <QScrollBar>


Alarm::Alarm(QWidget *parent,  SysTray *syst) : QWidget(parent), QTangoComProxyReader(this)
{

	ui.setupUi(this);
	d_systray = syst;
	QString soundServerName, alarmSysTestServerName;
	QStringList args = qApp->arguments();
	std::string s_instanceName;
	if(args.size() <= 1)
	{
		QString s;
		s += "error: tango device name requested as first argument\n";
		s += "example: " + args[0] + "alarms/test/1\n";
		s += "Correct TANGO_HOST environment variable has to be set too.\n";
		s += "If you want to enable sounds specify the tango sound device server like this:\n";
		s += "--sound-server-name $HOSTNAME/audio/card0\n";
		s += "To set style: style=fermi or style=booster\n";
		s += "To test alarms: --alarm-test-devname test/alarmtest/0 (or whatever device)\n";
		s += "Log database host: --log-database-host srv-log-srf (or whatever db host)\n";
		QMessageBox::information(this, "Usage", s);
		exit(EXIT_FAILURE);
	}
	device_name = args[1];

	connect(qtangoComHandle(), SIGNAL(newData(const TVariant&)), this, SLOT(refresh(const TVariant&))); //qui e' dove il nuovo allarme scatena il maeccanismo

	connect(ui.pBAckAll, SIGNAL(clicked()), this, SLOT(ackAll()));

	/* silence alarm */
	connect(ui.treeWidgetAlarms, SIGNAL(silence()), this, SLOT(silence()));
	connect(ui.treeWidgetAlarms, SIGNAL(silence()), this, SLOT(stopSound()));

	connect(ui.treeWidgetInternalErrors, SIGNAL(silence()), this, SLOT(silence()));
	connect(ui.treeWidgetInternalErrors, SIGNAL(silence()), this, SLOT(stopSound()));

	/* ack. Also uses QTWriter to impart StopNew command */
	connect(ui.treeWidgetAlarms, SIGNAL(ack()), this, SLOT(ack()));
	connect(ui.treeWidgetInternalErrors, SIGNAL(ack()), this, SLOT(ack()));

	connect(ui.pBApplyFilter, SIGNAL(clicked()), this, SLOT(applyFilter()));
	connect(ui.pBClearFilter, SIGNAL(clicked()), this, SLOT(clearFilter()));
	connect(ui.pBStopSound, SIGNAL(clicked()), this, SLOT(stopSound()));
	connect(ui.tabWidget, SIGNAL(currentChanged(int)), this, SLOT(tabChanged(int)));


	/* message label */
	//ui.msgLabel->setVisible(false);
	setLabelVisible(false);
	setLabelVisible(true);
	connect(ui.treeWidgetAlarms, SIGNAL(message(QString)), ui.msgLabel, SLOT(setText(QString)));
	connect(ui.treeWidgetInternalErrors, SIGNAL(message(QString)), ui.msgLabelInternal, SLOT(setText(QString)));
	//connect(ui.treeWidgetAlarms, SIGNAL(mouseOverItem(bool)), ui.msgLabel, SLOT(setVisible(bool)));
	//connect(ui.treeWidgetInternalErrors, SIGNAL(mouseOverItem(bool)), ui.msgLabelInternal, SLOT(setVisible(bool)));
	connect(ui.treeWidgetAlarms, SIGNAL(mouseOverItem(bool)), this, SLOT(setLabelVisible(bool)));
	connect(ui.treeWidgetInternalErrors, SIGNAL(mouseOverItem(bool)), this, SLOT(setLabelVisible(bool)));

/* connect the EUniqueApplication's `Rise()' signal to the
	* window's slot `raise()'.
	*/
	QObject::connect(qApp, SIGNAL(Rise() ), this, SLOT(raiseFromDbus() ) );



#ifdef QTANGOLIB_GET_VERSION
	enableHideEvents(false);
#else
	setHideEventEnabled(false);
#endif
	setSource(device_name + "/alarm");
	ui.pBClearFilter->setDisabled(true);
	ui.tWGroup->sortItems(0, Qt::AscendingOrder);
	ui.tWGroup->header()->setHidden(true);
	ui.pBConfHistory->setHidden(true);

	/* fill in the tree widget for the list of the states */
	try{
		Tango::Database *db = new Tango::Database();
		Tango::DbData dbd;
		dbd.push_back(Tango::DbDatum("GroupNames"));
		dbd.push_back(Tango::DbDatum("InstanceName"));
		vector<string> gnames;
		db->get_device_property(device_name.toStdString(), dbd);
		dbd[0] >> gnames;
		dbd[1] >> s_instanceName;
		instance_name = QString::fromStdString(s_instanceName);
		delete db;
		QStringList statesList;
		for(unsigned int i = 0; i < gnames.size(); i++)
			statesList << QString::fromStdString(gnames[i]).remove("gr_");
		statesList << "all";
		for(int i = 0; i < statesList.size(); i++){
			QTreeWidgetItem *item = new QTreeWidgetItem(ui.tWGroup, QStringList() << statesList[i]); //TODO: dove va il nuovo "item" ??
		}
	}
	catch(Tango::DevFailed &e)
	{
		TLog l(e);
		QMessageBox::critical(this, "Error connecting to the database",
				l.formatMessage());
		exit(EXIT_FAILURE);
	}



//---------------------------------------------------------------------------------------
	if(args.contains("--alarm-test-devname") && args.size() > args.indexOf("--alarm-test-devname") + 1)
	{
		alarmSysTestServerName = args.at(args.indexOf("--alarm-test-devname") + 1);
		printf("\e[1;32m --alarm-test-devname specified: \"%s\"\e[0m\n", qstoc(alarmSysTestServerName));
		ui.tPBTestAlarm->setTargets(alarmSysTestServerName + "/alarm_test(true)");
		ui.tPBTestAlarmStop->setTargets(alarmSysTestServerName + "/alarm_test(false)");
	}
	else
	{
		printf("no --alarm-test-devname specified: using ecrfpsa/alarmtest/0/alarm_test\n");
		ui.tPBTestAlarm->setTargets("ecrfpsa/alarmtest/0/alarm_test(true)");
		ui.tPBTestAlarmStop->setTargets("ecrfpsa/alarmtest/0/alarm_test(false)");
	}
//---------------------------------------------------------------------------------------


	if(args.contains("--sound-file-name") && args.size() > args.indexOf("--sound-file-name"))
	{
		soundServerName = args.at(args.indexOf("--sound-file-name") + 1);
		std::cout<<"Sound server name: "<<soundServerName.toStdString()<<std::endl;
	}
	else
		std::cout<<"if you want to enable sounds for alarms, provide as second argument the name of the sound file"<<std::endl;
	sound = new Sound(soundServerName);
	connect( ui.pBTestSoundOn,SIGNAL(clicked()), sound, SLOT(loop()));
	connect( ui.pBTestSoundOff,SIGNAL(clicked()), sound, SLOT(stop()));
	/* connections for History */
	connect(ui.pBApplyHistory, SIGNAL(clicked()), this, SLOT(queryDB()));
	ui.dateStart->setDate(QDate::currentDate());
	ui.dateStop->setDate(QDate::currentDate());
	ui.timeStart->setTime(QTime::currentTime());
	ui.timeStop->setTime(QTime::currentTime());
	ui.groupBoxFilter->setHidden(true);
	connect(ui.comboPeriod, SIGNAL(currentIndexChanged(int)), this, SLOT(enableFrameHistoryInterval(int)));
	connect(ui.pBFilter, SIGNAL(toggled(bool)), ui.groupBoxFilter, SLOT(setVisible(bool)));

	/* connections for filter group box */
	connect(ui.cBAck, SIGNAL(toggled(bool)), ui.rBAck, SLOT(setEnabled(bool)));
	connect(ui.cBAck, SIGNAL(toggled(bool)), ui.rBNack, SLOT(setEnabled(bool)));
	connect(ui.cBLevel, SIGNAL(toggled(bool)), ui.comboLog, SLOT(setEnabled(bool)));
	connect(ui.cBStatus, SIGNAL(toggled(bool)), ui.comboStatus, SLOT(setEnabled(bool)));
	connect(ui.cBGroup, SIGNAL(toggled(bool)), ui.tWGroup, SLOT(setEnabled(bool)));
	/* connections for test section */
//connect(ui.pBTestSoundOn, SIGNAL(clicked()), this, SLOT(testSound()));
//connect(ui.pBTestSoundOff, SIGNAL(clicked()), this, SLOT(stopSound()));
}

Alarm::~Alarm()
{

	if(sound)
	{
		sound->stop();
		delete sound;
	}
}

void Alarm::refresh(const TVariant &v)
{
	QString line;
	/* remove the filter */
	if (v.quality() == ATTR_INVALID)
	{
		ui.treeWidgetAlarms->setDisabled(true);
	}
	else if(v.canConvertToStringVector())
	{
		ui.treeWidgetAlarms->setDisabled(false);
	


		int scrollBarPos;
		QStringList fields;
		QStringList ids;
		AlarmTreeWidgetItem* item = NULL;

		QVector<QString> data = v.toStringVector();
	//printf("\e[1;35mdata size (nlines:) %d\e[0m\n", data.size());
		/* add items to the tree(s) if not already present */
	
		/* block signals to speed up updates */

		{
			ui.treeWidgetAlarms->blockSignals(true);
			ui.treeWidgetAlarms->model()->blockSignals(true);
			ui.treeWidgetAlarms->setUpdatesEnabled(false);
		}
		for (int i = data.size() - 1; i >= 0; i--)
		{
			line = data[i];
			fields = line.split("\t");
			playSound(fields);

	/*
			* add the device name (identifier of the alarm) to the list
			* of the current alarms
	*/
			ids << fields.at(AlarmTreeWidgetItem::Alarm);

			/* select third column : alarm name for comparison */
			item =  mItemsHash.value(fields.at(AlarmTreeWidgetItem::Alarm));
			/* save current scroll bar value */
			scrollBarPos = ui.treeWidgetAlarms->verticalScrollBar()->value();
			if(!item) /* create a new item */
			{
		//printf("add item number: %d\n", i);
				ui.treeWidgetAlarms->blockSignals(false);
				ui.treeWidgetAlarms->model()->blockSignals(false);
				addItem(fields);
				ui.treeWidgetAlarms->update(); //if not called new items are not shown
				ui.treeWidgetAlarms->blockSignals(true);
				ui.treeWidgetAlarms->model()->blockSignals(true);
			}
			else /* update fields of existing items */
			{
		//printf("update fields number: %d\n", i);
				item->update(fields);
			}
	
			/* restore scrollbar value */
			ui.treeWidgetAlarms->verticalScrollBar()->setValue(scrollBarPos);
		}

		/* remove block signals */

		{
			ui.treeWidgetAlarms->blockSignals(false);
			ui.treeWidgetAlarms->model()->blockSignals(false);
			ui.treeWidgetAlarms->setUpdatesEnabled(true);
		}


		/* after items have been added, we have to remove stale items */
	
		mRemoveStaleItems(ids);

		if(filtering && ui.cBDynamic->isChecked())
			applyFilter();

		/* update tree */
		ui.treeWidgetAlarms->update();
	
	}
}

/* called with signals blocked for the tree and the model
*/
void Alarm::addItem(const QStringList &list)
{
	AlarmTreeWidgetItem *item;
	QStringList fields(list);
	bool isNew = false;
	if(fields.size() > 0)
		fields[AlarmTreeWidgetItem::DateTime] =
				QDateTime::fromTime_t(list.at(AlarmTreeWidgetItem::DateTime).toUInt()).toString();

	if(fields.at(AlarmTreeWidgetItem::Alarm).contains("internal_error_"))
		item = new AlarmTreeWidgetItem(ui.treeWidgetInternalErrors, fields);
	else
	{
	/* done already by AlarmWidgetTreeItem (?)
		if(fields.last() == "NEW"){
		isNew = true;
		fields.removeLast(); // the last element contains info about "NEW", than must not be displayed
	}
	*/
		item = new AlarmTreeWidgetItem(ui.treeWidgetAlarms, fields);
		item->setNew(isNew);
	}

	/* add the couple identifier/AlarmTreeWidgetItem to the hash */
	mItemsHash.insert(fields.at(AlarmTreeWidgetItem::Alarm), item);
}

void Alarm::mRemoveStaleItems(const QStringList& currentIds)
{
	foreach(QString id, mItemsHash.keys())
	{
		if(!currentIds.contains(id))
		{
			AlarmTreeWidgetItem *item = mItemsHash.value(id);
			if(item)
			{
				mItemsHash.remove(id);
				delete item;
			}
		}
	}
}

QList<QTreeWidgetItem *> Alarm::mGetSelectedItems()
{
	QList<QTreeWidgetItem *> selItems;
	/* get the tree widget that is currently visible */
	if(ui.treeWidgetAlarms->isVisible())
		selItems = ui.treeWidgetAlarms->selectedItems();
	else if(ui.treeWidgetInternalErrors->isVisible())
		selItems = ui.treeWidgetInternalErrors->selectedItems();
	return selItems;
}

void Alarm::silence()
{
	QList<QTreeWidgetItem *> selItems = mGetSelectedItems();
	vector<string> toSilenceAlarms;
	if(selItems.size() == 0)
		QMessageBox::information(this, "Information", "Select one or more items with the\n"
				"mouse and then click again.\nMultiple selections are possible: just\n"
						"hold the control key while selecting the items with the mouse.");
	else
	{
		for(int i = 0; i < selItems.size(); i++)
		{
			QTreeWidgetItem *it = selItems[i];
			if(it->columnCount() > AlarmTreeWidgetItem::Alarm)
				toSilenceAlarms.push_back(it->text(AlarmTreeWidgetItem::Alarm).toStdString());
		}
		tangoStringListCommand(toSilenceAlarms, "Silence");
	}
}

AlarmTreeWidgetItem* Alarm::mGetItem(const QStringList& sl) const
{
	if(sl.size() < 3)
		return NULL;
	const QString id = sl.at(2);
	printf("size mappa %d\n", mItemsHash.size());
	return mItemsHash.value(id);
}

void Alarm::testSound()
{
	sound->loop();
}

void Alarm::playSound(const QStringList &splitted)
{
	if(splitted.size() > 5 && splitted[3] == "ALARM" && splitted.last()  == "NEW")
	{
		showNormal();
		sound->loop();
		d_systray->showMessage("Alarm detected", "An alarm has been detected.\n"
				"To stop the sound, click on the \"Stop sound\" button in the application\n"
						"Check the new alarm in the list, and click on this message to hide the baloon",
QSystemTrayIcon::Critical, 60000);
	}


}

void Alarm::stopSound()
{

	sound->stop();
}

void Alarm::ack()
{
	QList<QTreeWidgetItem *> selItems = mGetSelectedItems();
	vector<string> ackedAlarms;
	if(selItems.size() == 0)
		QMessageBox::information(this, "Information", "Select one or more items with the\n"
				"mouse and then click again.\nMultiple selections are possible: just\n"
						"hold the control key while selecting the items with the mouse.");
	else
	{
		for(int i = 0; i < selItems.size(); i++)
		{
			QTreeWidgetItem *it = selItems[i];
			if(it->columnCount() > AlarmTreeWidgetItem::Alarm)
				ackedAlarms.push_back(it->text(AlarmTreeWidgetItem::Alarm).toStdString());
		}
		tangoStringListCommand(ackedAlarms, "ack");
	}
	/* stop new command, just like it was in the previous version by means of TPushButton */
	QTWriter *stopNewWriter = findChild<QTWriter *>("stopNewWriter");
	if(!stopNewWriter)
	{
		stopNewWriter = new QTWriter(this);
		stopNewWriter->setObjectName("stopNewWriter");
		stopNewWriter->setTargets("$1->StopNew");
	}
	stopNewWriter->execute();
}

void Alarm::ackAll()
{
	QList<QTreeWidgetItem *>items;
	vector<string> ackedAlarms;
	/* a method to gather all items is search on wildcard */
	if(ui.treeWidgetAlarms->isVisible())
		items = ui.treeWidgetAlarms->findItems("*", Qt::MatchWildcard);
	else
		items = ui.treeWidgetInternalErrors->findItems("*", Qt::MatchWildcard);
	if(items.size() > 0)
	{
		for(int i = 0; i < items.size(); i++)
		{
			QTreeWidgetItem *it = items[i];
			if(it->columnCount() > AlarmTreeWidgetItem::Alarm)
				ackedAlarms.push_back(it->text(AlarmTreeWidgetItem::Alarm).toStdString());
		}
		tangoStringListCommand(ackedAlarms, "ack");
	}
	else
		QMessageBox::information(this, "Information", "No items are present on the list.\n"
				"There must be at least one item in the window to be\n"
						"able to remove it.");

}

void Alarm::tangoStringListCommand(vector<string> &names, const char *cmdname)
{
	Tango::DeviceProxy *dev = NULL;
	string devname = device_name.toStdString();
	Tango::DevVarStringArray data;
	data << names;
	Tango::DeviceData dd;
	dd << names;
	try
	{
		dev = new Tango::DeviceProxy(devname);
		dev->command_inout(cmdname,dd);
		delete dev;
	}
	catch(Tango::DevFailed &e)
	{
		QMessageBox::information(this, "Error sending the \"%1\" command",
					QString("Error sending the command \"ack\" to the tango device\n"
							"\"%2\"\nThe problem was:\n%3").
							arg(cmdname).
							arg(device_name).arg(tangoException(e)));
	}
}

void Alarm::applyFilter()
{
	AlarmFilter filter;

	filter.ack_enable = ui.cBAck->isChecked();

	if(filter.ack_enable)
		filter.ack = ui.rBAck->isChecked();

	filter.lev = ui.cBLevel->isChecked();
	if(filter.lev)
		filter.level = ui.comboLog->currentText();

	filter.stat = ui.cBStatus->isChecked();
	if(filter.stat)
		filter.status = ui.comboStatus->currentText();

	filter.grp = ui.cBGroup->isChecked();
	if(filter.grp)
	{
		QList<QTreeWidgetItem* >groups = ui.tWGroup->selectedItems();
		for(int i = 0; i < groups.size(); i++)
			filter.groups << groups[i]->text(0);
	}
	ui.treeWidgetAlarms->applyFilter(filter);
	ui.tWHistory->applyFilter(filter);
	filtering = true;
	ui.pBClearFilter->setDisabled(false);
}

void Alarm::clearFilter()
{
	ui.treeWidgetAlarms->removeFilter();
	ui.tWHistory->removeFilter();
	ui.pBClearFilter->setDisabled(true);
	ui.cBDynamic->setChecked(false);
	filtering = false;
}

QString Alarm::tangoException(Tango::DevFailed &e)
{
	QString ret;
	string err, descr, orig;
	int i;
	for(i = e.errors.length() - 1; i >= 0;  i--)
	{
		err = e.errors[i].reason;
		descr = e.errors[i].desc;
		orig = e.errors[i].origin;
		ret += QString("Level %1.\t%2\n\t%3\n\t%4\n\n").arg(i+1).arg(QString::fromStdString(err)).arg(QString::fromStdString(descr)).
				arg(QString::fromStdString(orig));
	}
	return ret;
}

void Alarm::queryDB()
{
	unsigned int startSearch;
	QDateTime now = QDateTime::currentDateTime();
	QString query;
	QDateTime dts, dte;
	switch(ui.comboPeriod->currentIndex())
	{
		case 0:
			startSearch = now.addDays(-1).toTime_t();
			query = QString("SELECT alarms.time_sec, alarms.time_usec, alarms.status, alarms.ack, description.name, description.level, "
					"description.grp, description.msg FROM "
					"alarms,description WHERE alarms.id_description=description.id_description AND description.instance='%1' AND  alarms.time_sec > %2 "
					"ORDER BY alarms.time_sec DESC").arg(instance_name).arg(startSearch);
			break;
		case 1:
			startSearch = now.addDays(-7).toTime_t();
			query = QString("SELECT alarms.time_sec, alarms.time_usec, alarms.status, alarms.ack, description.name, description.level, "
					"description.grp, description.msg FROM "
					"alarms,description WHERE alarms.id_description=description.id_description AND description.instance='%1' AND alarms.time_sec > %2 "
					"ORDER BY alarms.time_sec DESC").arg(instance_name).arg(startSearch);

			break;
		case 2:
			startSearch = now.addMonths(-1).toTime_t();
			query = QString("SELECT alarms.time_sec, alarms.time_usec, alarms.status, alarms.ack, description.name, description.level, "
					"description.grp, description.msg FROM "
					"alarms,description WHERE alarms.id_description=description.id_description AND description.instance='%1' AND alarms.time_sec > %2 "
					"ORDER BY alarms.time_sec DESC").arg(instance_name).arg(startSearch);

			break;
		case 3:
			dts.setDate(ui.dateStart->date());
			dts.setTime(ui.timeStart->time());
			dte.setDate(ui.dateStop->date());
			dte.setTime(ui.timeStop->time());
			query = QString("SELECT alarms.time_sec, alarms.time_usec, alarms.status, alarms.ack, description.name, description.level, "
					"description.grp, description.msg FROM "
					"alarms,description WHERE alarms.id_description=description.id_description AND description.instance='%1' AND alarms.time_sec > %2 AND alarms.time_sec < %3 "
					"ORDER BY alarms.time_sec DESC").arg(instance_name).arg(dts.toTime_t()).arg(dte.toTime_t());
			break;
		case 4:
			query = QString("SELECT alarms.time_sec, alarms.time_usec, alarms.status, alarms.ack, description.name, description.level, "
					"description.grp, description.msg FROM "
					"alarms,description WHERE alarms.id_description=description.id_description  AND description.instance='%1' "
					"ORDER BY alarms.time_sec DESC").arg(instance_name);

			break;
		default:
			break;
	}
	ui.tWHistory->executeQuery(query);
	if(ui.cBDynamic->isChecked())
	{
		applyFilter();
	}
}

void Alarm::enableFrameHistoryInterval(int i)
{
	if(i == 3)
		ui.widgetHystoryConf->setEnabled(true);
	else
		ui.widgetHystoryConf->setEnabled(false);
}

void Alarm::raiseFromDbus()
{
	hide();
	raise();
	show();

}

void Alarm::tabChanged(int t)
{
	if(t == 3)
	{
		ui.pBAckAll->setHidden(true);
		ui.pBConfHistory->setHidden(false);
		ui.pBConfHistory->setChecked(true);
		ui.groupBoxFilter->setDisabled(true);
		ui.pBFilter->setHidden(true);
		ui.pBConfHistory->setHidden(true);
		ui.pBStopSound->setHidden(true);
	}
	else if(t == 0 || t == 1)
	{
		ui.pBStopSound->setHidden(false);
		ui.pBAckAll->setHidden(false);
		ui.pBConfHistory->setHidden(true);
		ui.groupBoxFilter->setDisabled(false);
		ui.pBFilter->setHidden(false);
	}
	else if(t == 2)
	{
		ui.pBFilter->setHidden(false);
		ui.pBStopSound->setHidden(true);
		ui.pBAckAll->setHidden(true);
		ui.pBConfHistory->setHidden(true);
		ui.groupBoxFilter->setDisabled(false);
	}
}

void Alarm::systrayActivated(QSystemTrayIcon::ActivationReason r)
{
	if(r == QSystemTrayIcon::Trigger)
	{
		if(isVisible())
			hide();
		else
		{
			showNormal();
		}
	}
}

void Alarm::closeEvent(QCloseEvent *e)
{
//	return QWidget::closeEvent(e);
	hide();
	d_systray->showMessage("Application minimized", QString("The application \"%1\" has been minimized here.\n"
			"To restore it, left click on this icon,\nto quit the application choose \"Quit\" from the context menu.").arg(qApp->arguments().at(0)));
	e->ignore();
}


void Alarm::setLabelVisible(bool visibility)
{
 //"clear"  the msgLabel is visible is false
 // do not use clear() function beacuse it triggers a size change of the label!
	if (!visibility) ui.msgLabel->setText(" ");
}


