/***************************************************************************
 *   Copyright (C) 2007 by Claudio Scafuri, Giacomo Strangolino   *
 *   claudio@hyo   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef alarm_H
#define alarm_H

#include "ui_alarm.h"
#include <qtangocore.h>
#include <com_proxy_reader.h>
#include <vector>
#include <QString>
#include <QHash>
#include <QMap>
#include "Sound.h"
#include "systray.h"

class AlarmFilter;
class AlarmSQLHandle;
class AlarmHistoryTree;
class SysTray;
class AlarmTreeWidgetItem;

class Alarm: public QWidget, public QTangoComProxyReader
{
   Q_OBJECT

public:
   Alarm(QWidget *, SysTray *);
   ~Alarm();

public slots:
   void systrayActivated(QSystemTrayIcon::ActivationReason);

protected slots:
   void refresh(const TVariant &);
   void ack();
   void ackAll();
   void silence();
   void applyFilter();
   void clearFilter();
   void stopSound();
   void testSound();
   void queryDB();
   void tabChanged(int);
   void enableFrameHistoryInterval(int);
   void raiseFromDbus();
   void setLabelVisible(bool);


protected:
   void playSound(const QStringList &s);
   void closeEvent(QCloseEvent *);

private:
   void addItem(const QStringList &s);

   void tangoStringListCommand(vector<string> &names, const char *cmdname);

   /* checks if the tree already contains the item related to the
    * string list passed as input. Returns true if the item is contained
    * false otherwise. The 3rd field is considered the identifier of the
    * alarm.
    */
   AlarmTreeWidgetItem *mGetItem(const QStringList& sl) const;

   /* removes stale items each time the refresh() method is called */
   void mRemoveStaleItems(const QStringList& currentIds);

   QList<QTreeWidgetItem *> mGetSelectedItems();


   QString tangoException(Tango::DevFailed &e);
   Ui::Alarm ui;
   QVector<QString> oldData;
   QString device_name, instance_name;
   Sound *sound;
   bool filtering;
   SysTray *d_systray;

   QHash<QString, AlarmTreeWidgetItem*> mItemsHash;
};


#endif
