#include "systray.h"
#include <stdio.h>
#include <QMenu>
#include <QApplication>
#include <QDebug>


SysTray::SysTray(QObject *parent) : QSystemTrayIcon(parent)
{
	QIcon icon;
	bool found = false;
    mMessageShown = false;
    connect(this, SIGNAL(messageClicked()), this, SLOT(setMessageClicked()));
	for(int i = 0; i < qApp->arguments().size(); i++)
	{
	  if(qApp->arguments().at(i).contains(QRegExp("style\\s*=?\\s*booster")))
	  {
		icon.addPixmap(QPixmap(":icons/booster-alarm.png"));
		found = true;
		break;
	  }
	  else if(qApp->arguments().at(i).contains(QRegExp("style\\s*=?\\s*elettra")))
	  {
		icon.addPixmap(QPixmap(":icons/elettra-alarm.png"));
		found = true;
		break;
	  }
	  else if(qApp->arguments().at(i).contains(QRegExp("style\\s*=?\\s*fermi")))
	  {
		icon.addPixmap(QPixmap(":icons/fermi-alarm.png"));
		found = true;
		break;
	  }
	}
	if(!found)
	{
	  qDebug()<<"No \"style=elettra|fermi|booster\" specified in the command line. Using default elettra pixmap for systray icon";
	  icon.addPixmap(QPixmap(":icons/elettra-alarm.png"));
	}
	setIcon(icon);
}

void SysTray::showMessage(const QString & title,
                 const QString & message,
                 MessageIcon icon,
                 int millisecondsTimeoutHint)
{
    if(!mMessageShown)
    {
        QSystemTrayIcon::showMessage(title, message, icon, millisecondsTimeoutHint);
        mMessageShown = true;
    }
}

void SysTray::setMessageClicked()
{
    printf("setMessageClicked()\n");
    mMessageShown = false;
}

void SysTray::attach(QWidget *par)
{
	d_parent = par;
	d_menu = new QMenu(par);
	connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), par, SLOT(systrayActivated(QSystemTrayIcon::ActivationReason)));
	show();
	d_menu->addAction("Quit Application", qApp, SLOT(quit()));
	connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(menu(QSystemTrayIcon::ActivationReason)));
}

void SysTray::menu(QSystemTrayIcon::ActivationReason r)
{
	if(r == QSystemTrayIcon::Context)
		d_menu->exec(QCursor::pos());
}


